# -*- mode: python; coding: utf-8 -*-
#
# Pigment Python tools
#
# Copyright © 2006, 2007 Fluendo Embedded S.L.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

from pypgmtools.graph.group import Group
from pypgmtools.graph.image import Image
from pypgmtools.timing import implicit
import pgm, os

TRANSITION_FADING = 0
ANIMATION_STATIC = 0

class ImageTransition(Group):


    def __init__(self, canvas, layer):
        Group.__init__(self, canvas, layer)
        self._visible = Image()
        self._visible.visible = True
        self._visible.opacity = 255
        self._visible.bg_a = 0
        self.add(self._visible)
        self._animated_visible = implicit.AnimatedObject(self._visible)
        self._animated_visible.setup_next_animations(transformation = implicit.SMOOTH)

        self._hidden = Image()
        self._hidden.visible = True
        self._hidden.opacity = 0
        self._hidden.bg_a = 0
        self.add(self._hidden)
        self._animated_hidden = implicit.AnimatedObject(self._hidden)
        self._animated_hidden.setup_next_animations(transformation = implicit.SMOOTH)

        self.transition = TRANSITION_FADING
        self.animation = ANIMATION_STATIC

    def current_image_path__set(self, path):
        self._animated_hidden.stop_animations()
        self._hidden.opacity = 0
        fd = os.open(path, os.O_RDONLY)
        self._animated_hidden.set_from_fd(fd, 1024)

        if self.transition == TRANSITION_FADING:
            self._animated_hidden.opacity = 255
            self._animated_visible.opacity = 0

        self._animated_hidden, self._animated_visible = self._animated_visible, self._animated_hidden
        self._hidden, self._visible = self._visible, self._hidden

